﻿#include "Display.h"

// Адреса строк дисплея
PROGMEM u8 Row_Adress[] = {0x00,0x40,0x10,0x50};
// Контрастность дисплея
u8 Contrast = LCD_CONTRAST_DEFAULT;
// Режим удвоенной высоты шрифта
u8 Double_Font_Height = 0;


// Отправляет байт команды на дисплей
void Display_Cmd (u8 cmd)
{
	// RS=0, отправка команды
	LCD_PORT &= ~(1 << LCD_RS);
	// Выбор дисплея на шине
	Serial_Bus_Chip_Select();
	// Настройка режима последовательного интерфейса (количество и порядок бит)
	Serial_Bus_Mode_Set(8, MSB_First);
	// Отправка команды
	Serial_Bus_Send_Data(cmd);
	// Завершение обмена данными с дисплеем
	Serial_Bus_Chip_DeSelect();
}


// Отправляет байт данных на дисплей
void Display_Dat (u8 dat)
{
	// RS=1, отправка данных
	LCD_PORT |= (1 << LCD_RS);
	// Выбор дисплея на шине
	Serial_Bus_Chip_Select();
	// Настройка режима последовательного интерфейса (количество и порядок бит)
	Serial_Bus_Mode_Set(8, MSB_First);
	// Отправка данных
	Serial_Bus_Send_Data(dat);
	// Завершение обмена данными с дисплеем
	Serial_Bus_Chip_DeSelect();
}


// Задаёт курсор для вывода символов
void Display_Cursor (u8 y, u8 x)
{
	// Защита от превышения максимально допустимых границ курсора
	if ((x >= LCD_WIDTH) || (y >= LCD_HEIGHT)) return;
	// Вычисление адреса курсора
	u8 cursor = (pgm_read_byte(&Row_Adress[y])) | x;
	// Отправка курсора в дисплей
	Display_Cmd(LCD_CMD_SET_DDRAM_ADDR | cursor);
	// Задержка после установки курсора
	_delay_us(30);
}


// Очищает дисплей
void Display_Clear (void)
{
	Display_Cmd(LCD_CMD_CLEAR_DISPLAY);
	_delay_ms(2);
}


// Выводит цифру на дисплей
void Display_Dig (u8 d)
{
	Display_Dat (d + '0');
	_delay_us(30);
}


// Выводит символ на дисплей по его ASCII-коду
void Display_Char (char c)
{
	Display_Dat (c);
	_delay_us(30);
}


// Вывод числа на дисплей
// Число беззнаковое 32-битное
void Display_Num (u32 n)
{
	// Указатель на строку
	char* Stroka;
	// Конвертируем число в строку, нацеливаем указатель на начало строки
	Stroka = utoa (n);
	// Передаём указатель в функцию вывода строки
	Display_Str (Stroka);
}


// Вывод числа на дисплей
// Число знаковое 32-битное
void Display_SNum (s32 n)
{
	// Указатель на строку
	char* Stroka;
	// Конвертируем число в строку, нацеливаем указатель на начало строки
	Stroka = itoa (n);
	// Передаём указатель в функцию вывода строки
	Display_Str (Stroka);
}


// Вывод строки расположенной в оперативной памяти в буфер дисплея
void Display_Str (char *s)
{
	// Пока не дойдём до последнего символа
	while (*s != 0)
	{
		// Выводим символ
		Display_Char(*s);
		// Двигаемся к следующему символу
		++s;
	}
}


// Вывод строки расположенной в памяти программ в буфер дисплея
void Display_Str_P (const char *s)
{
	// Пока не дойдём до последнего символа
	while (pgm_read_byte(s) != 0)
	{
		// Выводим символ
		Display_Char(pgm_read_byte(s));
		// Двигаемся к следующему символу
		++s;
	}
}


// Настройка контрастности дисплея
// Может принимать значения от 0 до 63
// По умолчанию настроена контрастность 40
void Display_Contrast (u8 c)
{
	// Защита от установки недопустимой контрастности
	if (c >= 64) return;
	// Сохранение нового значения контрастности
	Contrast = c;
	// Обновление параметров дисплея
	Display_SW_Init();
}


// Включение удвоенной высоты шрифта
// 0 = режим 2 строки по 16 символов, обычный шрифт 5x8 (по умолчанию)
// 1 = режим 1 строка на 16 символов, высокий шрифт 5x16
void Display_Double_Font_Height (u8 d)
{
	// Сохранение новой высоты шрифта
	Double_Font_Height = d;
	// Обновление параметров дисплея
	Display_SW_Init();
}


// Инициализация дисплея
// Используется при изменении настроек дисплея
void Display_SW_Init (void)
{
	Display_Cmd(LCD_CMD_FUNCTION_SET | LCD_CMD_FUNCTION_SET_8BIT | LCD_CMD_FUNCTION_SET_1LINES | LCD_CMD_FUNCTION_SET_FONT5X8);
	_delay_us(30);
	Display_Cmd(LCD_CMD_FUNCTION_SET | LCD_CMD_FUNCTION_SET_8BIT | LCD_CMD_FUNCTION_SET_2LINES | LCD_CMD_FUNCTION_SET_FONT5X8 | LCD_CMD_FUNCTION_SET_IS);
	_delay_us(30);
	// Если включен режим удвоенной высоты шрифта
	if (Double_Font_Height)
	{
		// Включаем удвоенный шрифт и однострочный режим
		Display_Cmd(LCD_CMD_FUNCTION_SET | LCD_CMD_FUNCTION_SET_8BIT | LCD_CMD_FUNCTION_SET_1LINES | LCD_CMD_FUNCTION_SET_FONT5X16 | LCD_CMD_FUNCTION_SET_IS);
		_delay_us(30);
	}
	Display_Cmd(LCD_CMD_OSC_FREQ | LCD_CMD_OSC_FREQ_BIAS_1_5 | LCD_CMD_OSC_FREQ_183Hz);
	_delay_us(30);
	Display_Cmd(LCD_CMD_CONTRAST_L | (Contrast & LCD_CMD_CONTRAST_L_MASK));
	_delay_us(30);
	Display_Cmd(LCD_CMD_PWR_CONTRAST_H | LCD_CMD_PWR_CONTRAST_H_ICON_ON | LCD_CMD_PWR_CONTRAST_H_BOOST_ON | ((Contrast & LCD_CMD_PWR_CONTRAST_H_MASK) >> 4));
	_delay_us(30);
	Display_Cmd(LCD_CMD_FOLLOWER_CTRL | LCD_CMD_FOLLOWER_CTRL_ON | LCD_CMD_FOLLOWER_CTRL_AMPL_2_00);
	_delay_us(30);
	Display_Cmd(LCD_CMD_ONOFF_CONTROL | LCD_CMD_ONOFF_CONTROL_DISPON);
	_delay_us(30);
	Display_Cmd(LCD_CMD_CLEAR_DISPLAY);
	_delay_ms(2);
	Display_Cmd(LCD_CMD_ENTRY_MODE_SET | LCD_CMD_ENTRY_MODE_SET_INCR);
	_delay_us(30);
}


// Инициализирует дисплей
void Display_Init (void)
{
	// Настройка выводов RESET и RS в режим выходов
	LCD_DDR |= ((1 << LCD_RESET) | (1 << LCD_RS));
	// Аппаратный сброс дисплея
	LCD_PORT &= ~(1 << LCD_RESET);
	_delay_ms(10);
	// Запуск дисплея и стабилизация питания
	LCD_PORT |= (1 << LCD_RESET);
	_delay_ms(40);
	// Инициализация дисплея
	Display_SW_Init();
}

